// fxsinterface.v
// David Rowe 5 Feb 2009
//
// Top level module for FXS interface CPLD

module fxsinterface(clk, rx, fs, drx, ss, gpio_sclk, 
                    gpio_sdo, gpio_sdi, gpio_ncs, gpio_reset, mod_sclk, 
                    mod_sdo, mod_sdi, mod_ncs, mod_reset, sin_cpld);
input        clk;           // 2.048MHz system clock
output       rx;            // output to RS232 UART rx
output       fs;            // TDM bus frame clock
input        drx;           // TDM bus input from FXS
output       ss;            // SS signal to uC

// SPI signals just buffered by the CPLD

input        gpio_sclk;
output       gpio_sdo;
input        gpio_sdi;
input        gpio_ncs;
input        gpio_reset;

output       mod_sclk;
input        mod_sdo;
output       mod_sdi;
output       mod_ncs;
output       mod_reset;

input        sin_cpld;

wire         unbuf_rx;

assign mod_sclk = gpio_sclk;
assign gpio_sdo = mod_sdo;
assign mod_sdi = gpio_sdi;
assign mod_ncs = gpio_ncs;
assign mod_reset = gpio_reset;

// Generate 8 kHz frame sync (FS) pulse from 2.048MHz clock

divide256 my_divide256(
.clk      (clk),
.out      (fs)
);

// Generate 8 bit not SS pulse from FS to drive Atmel SPI port

genss my_genss(
.rst      (!gpio_reset),
.clk      (clk),
.fs       (fs),
.ss       (ss)
);

// RX TDM to UART converter

receive my_receive(
.rst      (!gpio_reset),
.clk      (clk),
.drx      (drx),
.fs       (fs),
.rx       (unbuf_rx)
);

/* 
  Use gpio_reset to control signal sent to UART RX pin.  This allows
  us to access the serial console during boot. If gpio_reset=0 route
  SIN_CPLD to RXD, this allows us to use the RS232 console.  If
  gpio_reset=1 we route TDM sample stream to the UART RX. 
*/

assign rx = gpio_reset ? unbuf_rx : sin_cpld;

endmodule

